""" User Views
"""
import copy

from core_dashboard_common_app import constants as dashboard_constants
from core_dashboard_common_app import settings
from core_dashboard_common_app.views.common.forms import UserForm
from core_main_app.components.blob import api as workspace_blob_api
from core_main_app.components.blob import utils as blob_utils
from core_main_app.components.data import api as workspace_data_api
from core_main_app.components.user import api as user_api
from core_main_app.components.workspace import api as workspace_api
from core_main_app.settings import INSTALLED_APPS
from core_main_app.utils.pagination.django_paginator.results_paginator import ResultsPaginator
from core_main_app.views.common.views import CommonView
from core_main_app.access_control.exceptions import AccessControlError


class DashboardWorkspaceTabs(CommonView):
    """ Workspace Tab Page
    """

    template = "core_dashboard_app/user/my_dashboard_container.html"
    data_template = "core_dashboard_app/common/list/my_dashboard_tabs.html"

    def get(self, request, workspace_id, *args, **kwargs):
        workspace = workspace_api.get_by_id(workspace_id)

        # Get the selected tab if given, otherwise data will be selected by default
        tab_selected = request.GET.get("tab", "data")
        items_to_render = []

        context = {}

        try:
            if tab_selected == "data":
                items_to_render = workspace_data_api.get_all_by_workspace(workspace, request.user)
                context.update(
                    {'document': dashboard_constants.FUNCTIONAL_OBJECT_ENUM.RECORD.value}
                )
            elif tab_selected == "file":
                items_to_render = workspace_blob_api.get_all_by_workspace(workspace, request.user)
                context.update({'document': dashboard_constants.FUNCTIONAL_OBJECT_ENUM.FILE.value})
        except AccessControlError as ace:
            items_to_render = []

        user_can_read = workspace_api.can_user_read_workspace(workspace, request.user)
        user_can_write = workspace_api.can_user_write_workspace(workspace, request.user)

        # Paginator
        page = request.GET.get('page', 1)
        results_paginator = ResultsPaginator.get_results(items_to_render, page, settings.RECORD_PER_PAGE_PAGINATION)

        # Data context
        results_paginator.object_list = self._format_document_context(request,
                                                                      results_paginator.object_list,
                                                                      user_can_read,
                                                                      user_can_write,
                                                                      tab_selected)

        # Add user_form for change owner
        user_form = UserForm(request.user)
        context.update({
            'workspace_id': workspace_id,
            'number_total': items_to_render.count(),
            'user_data': results_paginator,
            'user_form': user_form,
            'template': self.data_template,
            'administration': self.administration,
            'username_list': user_api.get_id_username_dict(user_api.get_all_users()),
            'tab': tab_selected,
            'title': workspace.title
        })

        # Get all username and corresponding ids
        user_names = dict((str(x.id), x.username) for x in user_api.get_all_users())
        context.update({'usernames': user_names})

        modals = [
                  dashboard_constants.MODALS_COMMON_CHANGE_OWNER,
                  dashboard_constants.MODALS_COMMON_DELETE
        ]

        assets = self._get_assets()

        if "core_file_preview_app" in INSTALLED_APPS:
            assets["js"].extend([
                {
                    "path": 'core_file_preview_app/user/js/file_preview.js',
                    "is_raw": False
                }
            ])
            assets["css"].append("core_file_preview_app/user/css/file_preview.css")
            modals.append("core_file_preview_app/user/file_preview_modal.html")

        return self.common_render(request,
                                  self.template,
                                  context=context,
                                  assets=assets,
                                  modals=modals)

    def _format_document_context(self, request, document_list, user_can_read, user_can_write, tab_selected):
        detailed_documents = []
        user = request.user
        for document in document_list:
            is_owner = str(document.user_id) == str(user.id) or self.administration
            document_context = {
                'can_read': user_can_read or is_owner,
                'can_write': user_can_write or is_owner
            }
            if tab_selected == "data":
                document_context.update({
                    'data': document,
                    'is_owner': is_owner
                })
            elif tab_selected == "file":
                document_context.update({
                    'file': document,
                    'url': blob_utils.get_blob_download_uri(document, request),
                    'user': user_api.get_user_by_id(document.user_id).username,
                    'date': document.id.generation_time,
                    'is_owner': is_owner
                })
            detailed_documents.append(document_context)
        return detailed_documents

    def _get_assets(self):
        assets = {
            "css": copy.deepcopy(dashboard_constants.CSS_COMMON),

            "js": [
                {
                    "path": 'core_main_app/user/js/workspaces/list/modals/assign_workspace.js',
                    "is_raw": False
                },
                {
                    "path": dashboard_constants.USER_VIEW_RECORD_RAW,
                    "is_raw": True
                },
                {
                    "path": dashboard_constants.JS_EDIT_RECORD,
                    "is_raw": False
                },
                {
                    "path": dashboard_constants.JS_USER_SELECTED_ELEMENT,
                    "is_raw": False
                },
                {
                    "path": dashboard_constants.JS_VIEW_RECORD,
                    "is_raw": False
                },
                {
                    "path": 'core_dashboard_common_app/user/js/init.raw.js',
                    "is_raw": True
                },
                {
                    "path": 'core_dashboard_common_app/common/js/init_pagination.js',
                    "is_raw": False
                },
                {
                    "path": dashboard_constants.JS_COMMON_FUNCTION_CHANGE_OWNER,
                    "is_raw": False
                },
                {
                    "path": dashboard_constants.JS_COMMON_FUNCTION_DELETE,
                    "is_raw": False
                },
                {
                    "path": 'core_dashboard_app/common/js/my_dashboard_tabs.js',
                    "is_raw": False
                },
            ]
        }

        assets['css'].append("core_dashboard_app/common/css/my_dashboard_tabs.css")

        if self.administration:
            assets['js'].append({
                "path": 'core_dashboard_app/admin/js/my_dashboard_tabs.raw.js',
                "is_raw": True
            })
        else:
            assets['js'].append({
                "path": 'core_dashboard_app/user/js/my_dashboard_tabs.raw.js',
                "is_raw": True
            })

        return assets



SECRET_KEY = 'fake-key'

INSTALLED_APPS = [
    # Django apps
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.sites',

    # Local apps
    "tests",
]

MIDDLEWARE = (
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
)

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [
            'templates'
        ],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
                'django.template.context_processors.i18n',
            ],
        },
    },
]



"""
    Url router for the user dashboard
"""
from django.contrib.auth.decorators import login_required
from django.urls import re_path
from django.urls import reverse_lazy

from core_dashboard_app.views.common import views as dashboard_app_common_views
from core_dashboard_common_app.views.common import ajax, views as dashboard_common_app_common_views
from core_dashboard_common_app.views.common.views import UserDashboardPasswordChangeFormView
from core_main_app.views.common.ajax import EditTemplateVersionManagerView

urlpatterns = [

    # Common
    re_path(r'^$', dashboard_common_app_common_views.home, name='core_dashboard_home'),
    re_path(r'^my-profile$', dashboard_common_app_common_views.my_profile, name='core_dashboard_profile'),
    re_path(r'^my-profile/edit$', dashboard_common_app_common_views.my_profile_edit,
            name='core_dashboard_profile_edit'),
    re_path(r'^my-profile/change-password', UserDashboardPasswordChangeFormView.as_view(
        template_name='core_dashboard_common_app/my_profile_change_password.html', success_url='/'),
            name='core_dashboard_profile_change_password'),

    re_path(r'^delete-document', ajax.delete_document, name='core_dashboard_delete_document'),
    re_path(r'^change-owner', ajax.change_owner_document, name='core_dashboard_change_owner_document'),
    re_path(r'^edit-record', ajax.edit_record, name='core_dashboard_edit_record'),

    # User
    re_path(r'^records$',
            login_required(dashboard_common_app_common_views.DashboardRecords.as_view(),
                           login_url=reverse_lazy("core_main_app_login")),
            name='core_dashboard_records'),
    re_path(r'^forms$',
            login_required(dashboard_common_app_common_views.DashboardForms.as_view(),
                           login_url=reverse_lazy("core_main_app_login")),
            name='core_dashboard_forms'),
    re_path(r'^templates$',
            login_required(dashboard_common_app_common_views.DashboardTemplates.as_view(),
                           login_url=reverse_lazy("core_main_app_login")),
            name='core_dashboard_templates'),
    re_path(r'^types$',
            login_required(dashboard_common_app_common_views.DashboardTypes.as_view(),
                           login_url=reverse_lazy("core_main_app_login")),
            name='core_dashboard_types'),
    re_path(r'^files$',
            login_required(dashboard_common_app_common_views.DashboardFiles.as_view(),
                           login_url=reverse_lazy("core_main_app_login")),
            name='core_dashboard_files'),
    re_path(r'^workspaces$',
            login_required(dashboard_common_app_common_views.DashboardWorkspaces.as_view(),
                           login_url=reverse_lazy("core_main_app_login")),
            name='core_dashboard_workspaces'),
    re_path(r'^workspace/(?P<workspace_id>\w+)$',
            login_required(dashboard_app_common_views.DashboardWorkspaceTabs.as_view(),
                           login_url=reverse_lazy("core_main_app_login")),
            name='core_dashboard_workspace_list'),
    re_path(r'^template/(?P<pk>[\w-]+)/edit/$',
            EditTemplateVersionManagerView.as_view(
                success_url=reverse_lazy("core_dashboard_templates")
            ),
            name='core_dashboard_app_edit_template'),
    re_path(r'^type/(?P<pk>[\w-]+)/edit/$',
            EditTemplateVersionManagerView.as_view(
                success_url=reverse_lazy("core_dashboard_types")
            ),
            name='core_dashboard_app_edit_type'),
]



"""
    Django settings for core_dashboard_app app
"""

from django.conf import settings

if not settings.configured:
    settings.configure()

SERVER_URI = getattr(settings, 'SERVER_URI', "http://localhost")

INSTALLED_APPS = getattr(settings, 'INSTALLED_APPS', [])



"""
    User Dashboard menu
"""
from django.urls import reverse
from menu import Menu, MenuItem

from core_dashboard_app.settings import INSTALLED_APPS
from core_dashboard_common_app.constants import FUNCTIONAL_OBJECT_ENUM

Menu.add_item(
    "dashboard", MenuItem('{0}s'.format(FUNCTIONAL_OBJECT_ENUM.TEMPLATE.value.title()), reverse('core_dashboard_templates'))
)

Menu.add_item(
    "dashboard", MenuItem('{0}s'.format(FUNCTIONAL_OBJECT_ENUM.RECORD.value.title()), reverse('core_dashboard_records'))
)

Menu.add_item(
    "dashboard", MenuItem('{0}s'.format(FUNCTIONAL_OBJECT_ENUM.FILE.value.title()), reverse('core_dashboard_files'))
)

Menu.add_item(
    "dashboard", MenuItem('{0}s'.format(FUNCTIONAL_OBJECT_ENUM.WORKSPACE.value.title()), reverse('core_dashboard_workspaces'))
)

sharing_children = (
    MenuItem('All {0}s'.format(FUNCTIONAL_OBJECT_ENUM.RECORD.value.title()), reverse("admin:core_dashboard_records"),
             icon="list"),
    MenuItem('All {0}s'.format(FUNCTIONAL_OBJECT_ENUM.TEMPLATE.value.title()), reverse("admin:core_dashboard_templates"),
             icon="list"),
    MenuItem('All {0}s'.format(FUNCTIONAL_OBJECT_ENUM.FILE.value.title()), reverse("admin:core_dashboard_files"),
             icon="list"),
    MenuItem('All {0}s'.format(FUNCTIONAL_OBJECT_ENUM.WORKSPACE.value.title()), reverse("admin:core_dashboard_workspaces"),
             icon="list"),
)

if 'core_composer_app' in INSTALLED_APPS:
    Menu.add_item(
        "dashboard",
        MenuItem('{0}s'.format(FUNCTIONAL_OBJECT_ENUM.TYPE.value.title()), reverse('core_dashboard_types'))
    )
    sharing_children += (MenuItem('All {0}s'.format(FUNCTIONAL_OBJECT_ENUM.TYPE.value.title()),
                                  reverse("admin:core_dashboard_types"), icon="list"),)

if 'core_curate_app' in INSTALLED_APPS:
    Menu.add_item(
        "dashboard",
        MenuItem('{0}s'.format(FUNCTIONAL_OBJECT_ENUM.FORM.value.title()), reverse('core_dashboard_forms'))
    )
    sharing_children += (MenuItem('All {0}s'.format(FUNCTIONAL_OBJECT_ENUM.FORM.value.title()),
                                  reverse("admin:core_dashboard_forms"), icon="list"),)

Menu.add_item(
    "user", MenuItem("My Profile", reverse('core_dashboard_profile'), icon="user")
)

Menu.add_item(
    "admin", MenuItem("DASHBOARD", None, children=sharing_children)
)
